@@ -160,6 +160,25 @@ SIM_FULL = Simulation(
             waist_sizes=(1.0, 2.0),
             waist_distances=(3.0, 4.0),
         ),
+        CustomFieldSource(
+            center=(1, 2, 3),
+            size=(2, 2, 0),
+            source_time=GaussianPulse(
+                freq0=2e14,
+                fwidth=4e13,
+            ),
+            field_dataset=FieldDataset(
+                Ex=ScalarFieldDataArray(
+                    np.ones((101, 101, 1, 1)),
+                    coords=dict(
+                        x=np.linspace(-1, 1, 101),
+                        y=np.linspace(-1, 1, 101),
+                        z=np.array([0]),
+                        f=[2e14],
+                    ),
+                )
+            ),
+        ),
     ],
     monitors=(
         FieldMonitor(
@@ -183,36 +202,46 @@ SIM_FULL = Simulation(
             freqs=[2e14, 2.5e14],
             mode_spec=ModeSpec(),
         ),
-        Near2FarAngleMonitor(
-            center=(1, 2, 3),
-            size=(2, 2, 2),
+        FieldProjectionAngleMonitor(
+            center=(0, 0, 0),
+            size=(0, 2, 2),
             freqs=[250e12, 300e12],
-            name="n2f_angle",
+            name="proj_angle",
             custom_origin=(1, 2, 3),
             phi=[0, np.pi / 2],
             theta=np.linspace(-np.pi / 2, np.pi / 2, 100),
         ),
-        Near2FarCartesianMonitor(
-            center=(1, 2, 3),
-            size=(2, 2, 2),
+        FieldProjectionCartesianMonitor(
+            center=(0, 0, 0),
+            size=(0, 2, 2),
             freqs=[250e12, 300e12],
-            name="n2f_cartesian",
+            name="proj_cartesian",
             custom_origin=(1, 2, 3),
             x=[-1, 0, 1],
             y=[-2, -1, 0, 1, 2],
-            plane_axis=2,
-            plane_distance=5,
+            proj_axis=2,
+            proj_distance=5,
+        ),
+        FieldProjectionKSpaceMonitor(
+            center=(0, 0, 0),
+            size=(0, 2, 2),
+            freqs=[250e12, 300e12],
+            name="proj_kspace",
+            custom_origin=(1, 2, 3),
+            proj_axis=2,
+            ux=[0.1, 0.2],
+            uy=[0.3, 0.4, 0.5],
+        ),
+        FieldProjectionAngleMonitor(
+            center=(0, 0, 0),
+            size=(0, 2, 2),
+            freqs=[250e12, 300e12],
+            name="proj_angle_exact",
+            custom_origin=(1, 2, 3),
+            phi=[0, np.pi / 2],
+            theta=np.linspace(-np.pi / 2, np.pi / 2, 100),
+            far_field_approx=False,
         ),
-        # Near2FarKSpaceMonitor(
-        #     center=(1,2,3),
-        #     size=(2,2,2),
-        #     freqs=[250e12, 300e12],
-        #     name='n2f_kspace',
-        #     custom_origin=(1,2,3),
-        #     u_axis=2,
-        #     ux=[1,2],
-        #     uy=[3,4,5]
-        # ),
         DiffractionMonitor(
             size=(0, inf, inf),
             center=(0, 0, 0),
@@ -223,7 +252,7 @@ SIM_FULL = Simulation(
     symmetry=(0, 0, 0),
     boundary_spec=BoundarySpec(
         x=Boundary(plus=PML(num_layers=20), minus=Absorber(num_layers=100)),
-        y=Boundary.bloch(bloch_vec=0.1),
+        y=Boundary.bloch(bloch_vec=1),
         z=Boundary.periodic(),
     ),
     shutoff=1e-6,
@@ -278,7 +307,7 @@ def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationDat
 
     def make_data(coords: dict, data_array_type: type, is_complex: bool = False) -> "data_type":
         """make a random DataArray out of supplied coordinates and data_type."""
-        data_shape = [len(coords[k]) for k in data_array_type.__slots__]
+        data_shape = [len(coords[k]) for k in data_array_type._dims]
         data = np.random.random(data_shape)
         data = (1 + 1j) * data if is_complex else data
         return data_array_type(data, coords=coords)
@@ -286,7 +315,7 @@ def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationDat
     def make_field_data(monitor: FieldMonitor) -> FieldData:
         """make a random FieldData from a FieldMonitor."""
         field_cmps = {}
-        coords = {"f": list(monitor.freqs)}
+        coords = {}
         grid = simulation.discretize(monitor, extend=True)
 
         for field_name in monitor.fields:
@@ -298,11 +327,18 @@ def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationDat
                 else:
                     coords[dim] = np.array(spatial_coords_dict[dim])
 
+            coords["f"] = list(monitor.freqs)
             field_cmps[field_name] = make_data(
                 coords=coords, data_array_type=ScalarFieldDataArray, is_complex=True
             )
 
-        return FieldData(monitor=monitor, **field_cmps)
+        return FieldData(
+            monitor=monitor,
+            symmetry=simulation.symmetry,
+            symmetry_center=simulation.center,
+            grid_expanded=simulation.discretize(monitor, extend=True),
+            **field_cmps
+        )
 
     def make_mode_data(monitor: ModeMonitor) -> ModeData:
         """make a random ModeData from a ModeMonitor."""
@@ -314,16 +350,16 @@ def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationDat
         n_complex = make_data(
             coords=coords_ind, data_array_type=ModeIndexDataArray, is_complex=True
         )
-        coords_amps = coords_ind.copy()
-        coords_amps["direction"] = ["+", "-"]
+        coords_amps = dict(direction=["+", "-"])
+        coords_amps.update(coords_ind)
         amps = make_data(coords=coords_amps, data_array_type=ModeAmpsDataArray, is_complex=True)
         return ModeData(monitor=monitor, n_complex=n_complex, amps=amps)
 
     MONITOR_MAKER_MAP = {FieldMonitor: make_field_data, ModeMonitor: make_mode_data}
 
-    montor_data = {mnt.name: MONITOR_MAKER_MAP[type(mnt)](mnt) for mnt in simulation.monitors}
+    data = [MONITOR_MAKER_MAP[type(mnt)](mnt) for mnt in simulation.monitors]
 
-    return SimulationData(simulation=simulation, monitor_data=montor_data)
+    return SimulationData(simulation=simulation, data=data)
 
 
 def assert_log_level(caplog, log_level_expected):
