@@ -8,11 +8,11 @@ import pydantic as pd
 import numpy as np
 
 from .base import Tidy3dBaseModel, cached_property
-from .types import Complex, Axis, Literal
+from .types import Complex, Axis, TYPE_TAG_STR
 from .source import GaussianBeam, ModeSource, PlaneWave
 from .medium import Medium
 
-from ..log import log, SetupError, DataError, ValidationError
+from ..log import log, SetupError, DataError
 from ..constants import EPSILON_0, MU_0, PML_SIGMA
 
 
@@ -51,7 +51,7 @@ class BlochBoundary(BoundaryEdge):
     >>> bloch = BlochBoundary(bloch_vec=1)
     """
 
-    bloch_vec: Complex = pd.Field(
+    bloch_vec: float = pd.Field(
         ...,
         title="Normalized Bloch vector component",
         description="Normalized component of the Bloch vector in units of "
@@ -143,13 +143,6 @@ class BlochBoundary(BoundaryEdge):
         bloch_vec = domain_size * k_global[axis]
         return cls(bloch_vec=bloch_vec)
 
-    @pd.validator("bloch_vec", always=True)
-    def _zero_imaginary_part(cls, val):
-        """Make sure the imaginary part of the bloch vector is 0."""
-        if np.imag(val) != 0:
-            raise ValidationError("Bloch vector must be real.")
-        return val
-
 
 # """ Absorber parameters """
 
@@ -335,7 +328,6 @@ PMLTypes = Union[PML, StablePML, Absorber, None]
 
 # types of boundaries that can be used in Simulation
 
-Symmetry = Literal[0, -1, 1]
 BoundaryEdgeType = Union[
     Periodic, PECBoundary, PMCBoundary, PML, StablePML, Absorber, BlochBoundary
 ]
@@ -353,12 +345,14 @@ class Boundary(Tidy3dBaseModel):
         Periodic(),
         title="Plus BC",
         description="Boundary condition on the plus side along a dimension.",
+        discriminator=TYPE_TAG_STR,
     )
 
     minus: BoundaryEdgeType = pd.Field(
         Periodic(),
         title="Minus BC",
         description="Boundary condition on the minus side along a dimension.",
+        discriminator=TYPE_TAG_STR,
     )
 
     @pd.root_validator(skip_on_failure=True)
