@@ -15,15 +15,17 @@ from .base import cached_property
 from .validators import assert_unique_names, assert_objects_in_sim_bounds
 from .validators import validate_mode_objects_symmetry
 from .geometry import Box
-from .types import Ax, Shapely, FreqBound, Axis, annotate_type
-from .grid import Coords1D, Grid, Coords, GridSpec, UniformGrid
+from .types import Ax, Shapely, FreqBound, Axis, annotate_type, Symmetry
+from .grid.grid import Coords1D, Grid, Coords
+from .grid.grid_spec import GridSpec, UniformGrid
 from .medium import Medium, MediumType, AbstractMedium, PECMedium
-from .boundary import BoundarySpec, Symmetry, BlochBoundary, PECBoundary, PMCBoundary, Periodic
-from .boundary import PML, StablePML, Absorber
+from .boundary import BoundarySpec, BlochBoundary, PECBoundary, PMCBoundary, Periodic
+from .boundary import PML, StablePML, Absorber, AbsorberSpec
 from .structure import Structure
-from .source import SourceType, PlaneWave, GaussianBeam, AstigmaticGaussianBeam
+from .source import SourceType, PlaneWave, GaussianBeam, AstigmaticGaussianBeam, CustomFieldSource
 from .monitor import MonitorType, Monitor, FreqMonitor
-from .monitor import AbstractFieldMonitor, DiffractionMonitor
+from .monitor import AbstractFieldMonitor, DiffractionMonitor, AbstractFieldProjectionMonitor
+from .data.dataset import Dataset
 from .viz import add_ax_if_none, equal_aspect
 
 from .viz import MEDIUM_CMAP, PlotParams, plot_params_symmetry
@@ -184,7 +186,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
     )
 
     courant: float = pydantic.Field(
-        0.9,
+        0.99,
         title="Courant Factor",
         description="Courant stability factor, controls time step to spatial step ratio. "
         "Lower values lead to more stable simulations for dispersive materials, "
@@ -240,6 +242,70 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 )
         return val
 
+    # pylint: disable=too-many-locals
+    @pydantic.validator("boundary_spec", always=True)
+    def plane_wave_boundaries(cls, val, values):
+        """Error if there are plane wave sources incompatible with boundary conditions."""
+        boundaries = val.to_list
+        sources = values.get("sources")
+        size = values.get("size")
+        sim_medium = values.get("medium")
+        structures = values.get("structures")
+        for src_idx, source in enumerate(sources):
+            if not isinstance(source, PlaneWave):
+                continue
+
+            _, tan_dirs = cls.pop_axis([0, 1, 2], axis=source.injection_axis)
+            medium_set = cls.intersecting_media(source, structures)
+            medium = medium_set.pop() if medium_set else sim_medium
+
+            for tan_dir in tan_dirs:
+                boundary = boundaries[tan_dir]
+
+                # check the PML/absorber + angled plane wave case
+                num_pml = sum(isinstance(bnd, AbsorberSpec) for bnd in boundary)
+                if num_pml > 0 and source.angle_theta != 0:
+                    raise SetupError(
+                        "Angled plane wave sources are not compatible with the absorbing boundary "
+                        f"along dimension {tan_dir}. Either set the source ``angle_theta`` to "
+                        "``0``, or use Bloch boundaries that match the source angle."
+                    )
+
+                # check the Bloch boundary + angled plane wave case
+                num_bloch = sum(isinstance(bnd, BlochBoundary) for bnd in boundary)
+                if num_bloch > 0:
+                    # make a dummy Bloch boundary to check for correctness
+                    dummy_bnd = BlochBoundary.from_source(
+                        source=source, domain_size=size[tan_dir], axis=tan_dir, medium=medium
+                    )
+                    expected_bloch_vec = dummy_bnd.bloch_vec
+
+                    if boundary[0].bloch_vec != expected_bloch_vec:
+                        test_val = np.abs(expected_bloch_vec - boundary[0].bloch_vec)
+
+                        if np.isclose(test_val % 1, 0) and not np.isclose(test_val, 0):
+                            # the given Bloch vector is offset by an integer
+                            log.warning(
+                                f"The wave vector of source at index {src_idx} along dimension "
+                                f"{tan_dir} is equal to the Bloch vector of the simulation "
+                                "boundaries along that dimension plus an integer reciprocal "
+                                "lattice vector. If using a ``DiffractionMonitor``, diffraction "
+                                "order 0 will not correspond to the angle of propagation "
+                                "of the source. Consider using ``BlochBoundary.from_source()``."
+                            )
+                        elif not np.isclose(test_val % 1, 0):
+                            # the given Bloch vector is neither equal to the expected value, nor
+                            # off by an integer
+                            log.warning(
+                                f"The Bloch vector along dimension {tan_dir} may be incorrectly "
+                                f"set with respect to the source at index {src_idx}. The absolute "
+                                "difference between the expected and provided values in "
+                                "bandstructure units, up to an integer offset, is greater than "
+                                "1e-6. Consider using ``BlochBoundary.from_source()``, or "
+                                "double-check that it was defined correctly."
+                            )
+        return val
+
     @pydantic.validator("structures", always=True)
     def _validate_num_mediums(cls, val):
         """Error if too many mediums present."""
@@ -438,6 +504,82 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                         )
         return val
 
+    # pylint: disable=too-many-locals
+    @pydantic.validator("monitors", always=True)
+    def _projection_monitors_homogeneous(cls, val, values):
+        """Error if any field projection monitor is not in a homogeneous region."""
+
+        if val is None:
+            return val
+
+        # list of structures including background as a Box()
+        structure_bg = Structure(
+            geometry=Box(
+                size=values.get("size"),
+                center=values.get("center"),
+            ),
+            medium=values.get("medium"),
+        )
+
+        structures = values.get("structures") or []
+        total_structures = [structure_bg] + list(structures)
+
+        for monitor in val:
+            if isinstance(monitor, (AbstractFieldProjectionMonitor, DiffractionMonitor)):
+                mediums = cls.intersecting_media(monitor, total_structures)
+                # make sure there is no more than one medium in the returned list
+                if len(mediums) > 1:
+                    raise SetupError(
+                        f"{len(mediums)} different mediums detected on plane "
+                        f"intersecting a {monitor.type}. Plane must be homogeneous."
+                    )
+
+        return val
+
+    @pydantic.validator("monitors", always=True)
+    def _projection_monitors_distance(cls, val, values):
+        """Warn if the projection distance is large for exact projections."""
+
+        if val is None:
+            return val
+
+        sim_size = values.get("size")
+
+        for idx, monitor in enumerate(val):
+            if isinstance(monitor, AbstractFieldProjectionMonitor):
+                if (
+                    np.abs(monitor.proj_distance) > 1.0e4 * np.max(sim_size)
+                    and not monitor.far_field_approx
+                ):
+                    monitor = monitor.copy(update={"far_field_approx": True})
+                    val = list(val)
+                    val[idx] = monitor
+                    val = tuple(val)
+                    log.warning(
+                        "A very large projection distance was set for the field projection "
+                        f"monitor '{monitor.name}'. Using exact field projections may result in "
+                        "precision loss for large distances; automatically enabling far-field "
+                        "approximations ('far_field_approx = True') for better precision. "
+                        "To insist on exact projections, consider using client-side projections "
+                        "via the 'FieldProjector' class, where higher precision is available."
+                    )
+        return val
+
+    @pydantic.validator("monitors", always=True)
+    def diffraction_monitor_medium(cls, val, values):
+        """If any :class:`DiffractionMonitor` exists, ensure is does not lie in a lossy medium."""
+        monitors = val
+        structures = values.get("structures")
+        medium = values.get("medium")
+        for monitor in monitors:
+            if isinstance(monitor, DiffractionMonitor):
+                medium_set = Simulation.intersecting_media(monitor, structures)
+                medium = medium_set.pop() if medium_set else medium
+                _, index_k = medium.nk_model(frequency=np.array(monitor.freqs))
+                if not np.all(index_k == 0):
+                    raise SetupError("Diffraction monitors must not lie in a lossy medium.")
+        return val
+
     @pydantic.validator("grid_spec", always=True)
     def _warn_grid_size_too_small(cls, val, values):  # pylint:disable=too-many-locals
         """Warn user if any grid size is too large compared to minimum wavelength in material."""
@@ -503,18 +645,9 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         # for each plane wave in the sources list
         for source in val:
             if isinstance(source, (PlaneWave, GaussianBeam, AstigmaticGaussianBeam)):
-
-                # get all merged structures on the plane
-                normal_axis_index = source.size.index(0.0)
-                dim = "xyz"[normal_axis_index]
-                pos = source.center[normal_axis_index]
-                xyz_kwargs = {dim: pos}
-                structures_merged = cls._filter_structures_plane(total_structures, **xyz_kwargs)
-
+                mediums = cls.intersecting_media(source, total_structures)
                 # make sure there is no more than one medium in the returned list
-                mediums = {medium for medium, _ in structures_merged}
                 if len(mediums) > 1:
-
                     raise SetupError(
                         f"{len(mediums)} different mediums detected on plane "
                         f"intersecting a {source.type} source. Plane must be homogeneous."
@@ -544,6 +677,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         """Validate the fully initialized simulation is ok for upload to our servers."""
         self._validate_size()
         self._validate_monitor_size()
+        self._validate_datasets_not_none()
         # self._validate_run_time()
 
     def _validate_size(self) -> None:
@@ -594,6 +728,15 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 f"a maximum of {MAX_MONITOR_DATA_SIZE_BYTES:.2e} are allowed."
             )
 
+    def _validate_datasets_not_none(self) -> None:
+        """Ensures that all custom datasets are defined."""
+        if any(dataset is None for dataset in self.custom_datasets):
+            raise SetupError(
+                "Data for a custom data component is missing. This can happen for example if the "
+                "Simulation has been loaded from json. To save and load simulations with custom "
+                "data, use hdf5 format instead."
+            )
+
     """ Accounting """
 
     @cached_property
@@ -635,6 +778,67 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         geometry = Box(size=(inf, inf, inf))
         return Structure(geometry=geometry, medium=self.medium)
 
+    @staticmethod
+    def intersecting_media(
+        test_object: Box, structures: Tuple[Structure, ...]
+    ) -> Tuple[MediumType, ...]:
+        """From a given list of structures, returns a list of :class:`AbstractMedium` associated
+        with those structures that intersect with the ``test_object``, if it is a surface, or its
+        surfaces, if it is a volume.
+
+        Parameters
+        -------
+        test_object : :class:`Box`
+            Object for which intersecting media are to be detected.
+        structures : List[:class:`AbstractMedium`]
+            List of structures whose media will be tested.
+
+        Returns
+        -------
+        List[:class:`AbstractMedium`]
+            Set of distinct mediums that intersect with the given planar object.
+        """
+        if test_object.size.count(0.0) == 1:
+            # get all merged structures on the test_object, which is already planar
+            normal_axis_index = test_object.size.index(0.0)
+            dim = "xyz"[normal_axis_index]
+            pos = test_object.center[normal_axis_index]
+            xyz_kwargs = {dim: pos}
+            structures_merged = Simulation._filter_structures_plane(structures, **xyz_kwargs)
+            mediums = {medium for medium, _ in structures_merged}
+            return mediums
+
+        # if the test object is a volume, test each surface recursively
+        object_dict = test_object.dict()
+        exclude_surfaces = object_dict.pop("exclude_surfaces", None)
+        surfaces = test_object.surfaces(**object_dict)
+        if exclude_surfaces:
+            surfaces = [surf for surf in surfaces if surf.name[-2:] not in exclude_surfaces]
+        mediums = set()
+        for surface in surfaces:
+            _mediums = Simulation.intersecting_media(surface, structures)
+            mediums.update(_mediums)
+        return mediums
+
+    def monitor_medium(self, monitor: MonitorType):
+        """Return the medium in which the given monitor resides.
+
+        Parameters
+        -------
+        monitor : :class:`Monitor`
+            Monitor whose associated medium is to be returned.
+
+        Returns
+        -------
+        :class:`AbstractMedium`
+            Medium associated with the given :class:`Monitor`.
+        """
+        medium_set = self.intersecting_media(monitor, self.structures)
+        if len(medium_set) > 1:
+            raise SetupError(f"Monitor {monitor.name} intersects more than one medium.")
+        medium = medium_set.pop() if medium_set else self.medium
+        return medium
+
     """ Plotting """
 
     @equal_aspect
@@ -1285,8 +1489,6 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
             The supplied or created matplotlib axes.
         """
 
-        kwargs.setdefault("clip_on", False)
-
         def set_plot_params(boundary_edge, lim, side, thickness):
             """Return the line plot properties such as color and opacity based on the boundary"""
             if isinstance(boundary_edge, PECBoundary):
@@ -1398,8 +1600,10 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         _, (xmin, ymin) = self.pop_axis(self.bounds_pml[0], axis=axis)
         _, (xmax, ymax) = self.pop_axis(self.bounds_pml[1], axis=axis)
 
-        ax.set_xlim(xmin, xmax)
-        ax.set_ylim(ymin, ymax)
+        if xmin != xmax:
+            ax.set_xlim(xmin, xmax)
+        if ymin != ymax:
+            ax.set_ylim(ymin, ymax)
         return ax
 
     @staticmethod
@@ -1670,28 +1874,29 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 # bmin <= center <= bmax
                 bmin_tmp = center
                 bmax_tmp = max(bmax, 2 * center - bmin)
-            # Extend well past the simulation domain if needed, but truncate if original box
-            # is too large, specifically to avoid issues with inf.
-            sim_size = sim_bmax - sim_bmin
-            bmin_new.append(max(bmin_tmp, sim_bmin - sim_size))
-            bmax_new.append(min(bmax_tmp, sim_bmax + sim_size))
+            # If inf, extend well past the simulation domain but make a finite number
+            if np.isinf(bmin_tmp):
+                bmin_tmp = sim_bmin - np.amax(self.size)
+            if np.isinf(bmax_tmp):
+                bmax_tmp = sim_bmax + np.amax(self.size)
+            bmin_new.append(bmin_tmp)
+            bmax_new.append(bmax_tmp)
 
         return Box.from_bounds(bmin_new, bmax_new)
 
-    def discretize(self, box: Box, extend: bool = False, snap_zero_dim: bool = False) -> Grid:
+    def discretize(self, box: Box, snap_zero_dim: bool = False, **kwargs) -> Grid:
         """Grid containing only cells that intersect with a :class:`Box`.
 
         Parameters
         ----------
         box : :class:`Box`
             Rectangular geometry within simulation to discretize.
-        extend : bool
-            If ``True``, extra pixels are added to the discretized grid if needed, such that Yee
-            grid fields can be interpolated to any point inside the ``box``.
         snap_zero_dim : bool
             If ``True``, and the ``box`` has size zero along a given direction, the ``grid`` is
             defined to also have a zero-sized cell exactly centered at the ``box`` center. If
             false, the ``simulation`` grid cell containing the ``box`` center is instead used.
+        kwargs :
+            Extra keyword arguments passed to ``discretize_inds`` method of :class:`.Grid`.
 
         Returns
         -------
@@ -1702,16 +1907,19 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         if not self.intersects(box):
             log.error(f"Box {box} is outside simulation, cannot discretize")
 
-        span_inds = self.grid.discretize_inds(box, extend=extend)
+        span_inds = self.grid.discretize_inds(box, **kwargs)
         boundary_dict = {}
         for idim, dim in enumerate("xyz"):
-            if snap_zero_dim and box.size[idim] == 0:
-                boundary_dict[dim] = [box.center[idim], box.center[idim]]
-            else:
-                ind_beg, ind_end = span_inds[idim]
-                # ind_end + 1 because we are selecting cell boundaries not cells
-                boundary_dict[dim] = self.grid.periodic_subspace(idim, ind_beg, ind_end + 1)
+            ind_beg, ind_end = span_inds[idim]
+            # ind_end + 1 because we are selecting cell boundaries not cells
+            boundary_dict[dim] = self.grid.periodic_subspace(idim, ind_beg, ind_end + 1)
 
+            # Overwrite with zero dimension snapped, if requested
+            if snap_zero_dim:
+                if self.size[idim] == 0:
+                    boundary_dict[dim] = [self.center[idim], self.center[idim]]
+                elif box.size[idim] == 0:
+                    boundary_dict[dim] = [box.center[idim], box.center[idim]]
         return Grid(boundaries=Coords(**boundary_dict))
 
     def epsilon(
@@ -1808,16 +2016,18 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         return make_eps_data(coords)
 
     @classmethod
-    def from_file(cls, fname: str, **parse_kwargs) -> Simulation:
+    def from_file(cls, fname: str, group_path: str = None, **parse_obj_kwargs) -> Simulation:
         """Loads a :class:`Tidy3dBaseModel` from .yaml or .json file.
 
         Parameters
         ----------
         fname : str
             Full path to the .yaml or .json file to load the :class:`Tidy3dBaseModel` from.
-        **parse_kwargs
-            Keyword arguments passed to either pydantic's ``parse_file`` or ``parse_raw`` methods
-            for ``.json`` and ``.yaml`` file formats, respectively.
+        group_path : str, optional
+            Path to a group inside the file to use as the base level. Only for ``.hdf5`` files.
+        **parse_obj_kwargs
+            Keyword arguments passed to either pydantic's ``parse_obj`` function when loading model.
+
         Returns
         -------
         :class:`Tidy3dBaseModel`
@@ -1828,6 +2038,15 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         >>> simulation = Simulation.from_file(fname='folder/sim.json') # doctest: +SKIP
         """
 
-        updater = Updater.from_file(fname)
-        sim_dict = updater.update_to_current()
-        return cls.parse_obj(sim_dict, **parse_kwargs)
+        sim_dict = cls.dict_from_file(fname=fname, group_path=group_path)
+        updater = Updater(sim_dict=sim_dict)
+        sim_dict_updated = updater.update_to_current()
+        return cls.parse_obj(sim_dict_updated, **parse_obj_kwargs)
+
+    @property
+    def custom_datasets(self) -> List[Dataset]:
+        """List of custom datasets for verification purposes. If the list is not empty, then
+        the simulation needs to be exported to hdf5 to store the data.
+        """
+        datasets = [src.field_dataset for src in self.sources if isinstance(src, CustomFieldSource)]
+        return datasets
