@@ -90,8 +90,10 @@ class Updater(pd.BaseModel):
     def from_file(cls, fname: str) -> Updater:
         """Dictionary representing the simulation loaded from file."""
 
+        # TODO: fix this, it broke
         if ".hdf5" in fname:
-            sim_dict = Tidy3dBaseModel.hdf5_to_dict(fname=fname)
+            sim_dict = Tidy3dBaseModel.from_file(fname=fname).dict()
+
         else:
             # try:
             with open(fname, "r", encoding="utf-8") as f:
@@ -200,6 +202,59 @@ def iterate_update_dict(update_dict: Dict, update_types: Dict[str, Callable]):
             iterate_update_dict(item, update_types)
 
 
+@updates_from_version("1.7")
+def update_1_7(sim_dict: dict) -> dict:
+    """Updates version 1.7."""
+
+    def fix_angle_info(mnt_dict: dict) -> dict:
+        mnt_dict["type"] = "FieldProjectionAngleMonitor"
+        mnt_dict.pop("fields")
+        mnt_dict.pop("medium")
+        mnt_dict["proj_distance"] = 1e6
+        return mnt_dict
+
+    def fix_cartesian_info(mnt_dict: dict) -> dict:
+        mnt_dict["type"] = "FieldProjectionCartesianMonitor"
+        mnt_dict.pop("fields")
+        mnt_dict.pop("medium")
+        dist = mnt_dict.pop("plane_distance")
+        mnt_dict["proj_distance"] = dist
+        axis = mnt_dict.pop("plane_axis")
+        mnt_dict["proj_axis"] = axis
+        return mnt_dict
+
+    def fix_kspace_info(mnt_dict: dict) -> dict:
+        mnt_dict["type"] = "FieldProjectionKSpaceMonitor"
+        mnt_dict.pop("fields")
+        mnt_dict.pop("medium")
+        mnt_dict["proj_distance"] = 1e6
+        axis = mnt_dict.pop("u_axis")
+        mnt_dict["proj_axis"] = axis
+        return mnt_dict
+
+    def fix_diffraction_info(mnt_dict: dict) -> dict:
+        mnt_dict.pop("medium", None)
+        mnt_dict.pop("orders_x", None)
+        mnt_dict.pop("orders_y", None)
+        return mnt_dict
+
+    def fix_bloch_vec(mnt_dict: dict) -> dict:
+        mnt_dict["bloch_vec"] = mnt_dict["bloch_vec"]["real"]
+        return mnt_dict
+
+    iterate_update_dict(
+        update_dict=sim_dict,
+        update_types={
+            "Near2FarAngleMonitor": fix_angle_info,
+            "Near2FarCartesianMonitor": fix_cartesian_info,
+            "Near2FarKSpaceMonitor": fix_kspace_info,
+            "DiffractionMonitor": fix_diffraction_info,
+            "BlochBoundary": fix_bloch_vec,
+        },
+    )
+    return sim_dict
+
+
 @updates_from_version("1.6")
 def update_1_6(sim_dict: dict) -> dict:
     """Updates version 1.6."""
