@@ -1,7 +1,7 @@
 """higher level wrappers for webapi functions for individual (Job) and batch (Batch) tasks."""
 import os
 from abc import ABC
-from typing import Dict, Optional, Tuple
+from typing import Dict, Tuple
 import time
 
 from rich.console import Console
@@ -11,8 +11,9 @@ import pydantic as pd
 from . import webapi as web
 from .task import TaskId, TaskInfo, RunInfo, TaskName
 from ..components.simulation import Simulation
-from ..components.data import SimulationData
 from ..components.base import Tidy3dBaseModel
+from ..components.data.sim_data import SimulationData
+
 from ..log import DataError
 
 
@@ -37,12 +38,6 @@ class Job(WebContainer):
         "default", title="Folder Name", description="Name of folder to store task on web UI."
     )
 
-    task_id: TaskId = pd.Field(
-        None,
-        title="Task Id",
-        description="Task ID number, set when the task is uploaded, leave as None.",
-    )
-
     callback_url: str = pd.Field(
         None,
         title="Callback URL",
@@ -51,20 +46,19 @@ class Job(WebContainer):
         "``{'id', 'status', 'name', 'workUnit', 'solverVersion'}``.",
     )
 
-    def run(
-        self, path: str = DEFAULT_DATA_PATH, normalize_index: Optional[int] = 0
-    ) -> SimulationData:
+    task_id: TaskId = pd.Field(
+        None,
+        title="Task Id",
+        description="Task ID number, set when the task is uploaded, leave as None.",
+    )
+
+    def run(self, path: str = DEFAULT_DATA_PATH) -> SimulationData:
         """run :class:`Job` all the way through and return data.
 
         Parameters
         ----------
         path_dir : str = "./simulation_data.hdf5"
             Base directory where data will be downloaded, by default current working directory.
-        normalize_index : int = 0
-            If specified, normalizes the frequency-domain data by the amplitude spectrum of the
-            source corresponding to ``simulation.sources[normalize_index]``.
-            This occurs when the data is loaded into a :class:`.SimulationData` object.
-            To turn off normalization, set ``normalize_index`` to ``None``.
 
         Returns
         -------
@@ -72,25 +66,23 @@ class Job(WebContainer):
             Dictionary mapping task name to :class:`.SimulationData` for :class:`Job`.
         """
 
-        self.upload()
         self.start()
         self.monitor()
-        return self.load(path=path, normalize_index=normalize_index)
+        return self.load(path=path)
 
-    def upload(self) -> None:
-        """Upload simulation to server without running.
+    @pd.validator("task_id", always=True)
+    def _upload(cls, val, values) -> None:
+        """Upload simulation to server without running."""
+        if val is not None:
+            return val
 
-        Note
-        ----
-        To start the simulation running, call :meth:`Job.start` after uploaded.
-        """
         task_id = web.upload(
-            simulation=self.simulation,
-            task_name=self.task_name,
-            folder_name=self.folder_name,
-            callback_url=self.callback_url,
+            simulation=values.get("simulation"),
+            task_name=values.get("task_name"),
+            folder_name=values.get("folder_name"),
+            callback_url=values.get("callback_url"),
         )
-        self.task_id = task_id
+        return task_id
 
     def get_info(self) -> TaskInfo:
         """Return information about a :class:`Job`.
@@ -101,8 +93,7 @@ class Job(WebContainer):
             :class:`TaskInfo` object containing info about status, size, credits of task and others.
         """
 
-        task_info = web.get_info(task_id=self.task_id)
-        return task_info
+        return web.get_info(task_id=self.task_id)
 
     @property
     def status(self):
@@ -126,8 +117,7 @@ class Job(WebContainer):
         :class:`RunInfo`
             Task run information.
         """
-        run_info = web.get_run_info(task_id=self.task_id)
-        return run_info
+        return web.get_run_info(task_id=self.task_id)
 
     def monitor(self) -> None:
         """Monitor progress of running :class:`Job`.
@@ -153,9 +143,7 @@ class Job(WebContainer):
         """
         web.download(task_id=self.task_id, path=path)
 
-    def load(
-        self, path: str = DEFAULT_DATA_PATH, normalize_index: Optional[int] = 0
-    ) -> SimulationData:
+    def load(self, path: str = DEFAULT_DATA_PATH) -> SimulationData:
         """Download results from simulation (if not already) and load them into ``SimulationData``
         object.
 
@@ -163,27 +151,17 @@ class Job(WebContainer):
         ----------
         path : str = "./simulation_data.hdf5"
             Path to download data as ``.hdf5`` file (including filename).
-        normalize_index : int = 0
-            If specified, normalizes the frequency-domain data by the amplitude spectrum of the
-            source corresponding to ``simulation.sources[normalize_index]``.
-            This occurs when the data is loaded into a :class:`.SimulationData` object.
-            To turn off normalization, set ``normalize_index`` to ``None``.
 
         Returns
         -------
         :class:`.SimulationData`
             Object containing data about simulation.
         """
-        return web.load(
-            task_id=self.task_id,
-            path=path,
-            normalize_index=normalize_index,
-        )
+        return web.load(task_id=self.task_id, path=path)
 
     def delete(self):
         """Delete server-side data associated with :class:`Job`."""
         web.delete(self.task_id)
-        self.task_id = None
 
 
 class BatchData(Tidy3dBaseModel):
@@ -199,14 +177,6 @@ class BatchData(Tidy3dBaseModel):
         ..., title="Task IDs", description="Mapping of task_name to task_id for each task in batch."
     )
 
-    normalize_index: Optional[int] = pd.Field(
-        0,
-        title="Source Normalization Index",
-        description="If specified, normalizes the frequency-domain data by the amplitude "
-        "spectrum of the source corresponding to ``simulation.sources[normalize_index]``. "
-        "If ``None``, does not normalize.",
-    )
-
     def load_sim_data(self, task_name: str) -> SimulationData:
         """Load a :class:`.SimulationData` from file by task name."""
         task_data_path = self.task_paths[task_name]
@@ -214,7 +184,6 @@ class BatchData(Tidy3dBaseModel):
         return web.load(
             task_id=task_id,
             path=task_data_path,
-            normalize_index=self.normalize_index,
             replace_existing=False,
         )
 
@@ -228,16 +197,14 @@ class BatchData(Tidy3dBaseModel):
         return self.load_sim_data(task_name)
 
     @classmethod
-    def load(
-        cls, path_dir: str = DEFAULT_DATA_DIR, normalize_index: Optional[int] = 0
-    ) -> "BatchData":
+    def load(cls, path_dir: str = DEFAULT_DATA_DIR) -> "BatchData":
         """Load :class:`Batch` from file, download results, and load them.
 
         Parameters
         ----------
         path_dir : str = './'
             Base directory where data will be downloaded, by default current working directory.
-            A `batch.json` file must be present in the directory.
+            A `batch.hdf5` file must be present in the directory.
 
         Returns
         ------
@@ -247,7 +214,7 @@ class BatchData(Tidy3dBaseModel):
 
         batch_file = Batch._batch_path(path_dir=path_dir)  # pylint:disable=protected-access
         batch = Batch.from_file(batch_file)
-        return batch.load(path_dir=path_dir, normalize_index=normalize_index)
+        return batch.load(path_dir=path_dir)
 
 
 class Batch(WebContainer):
@@ -259,6 +226,12 @@ class Batch(WebContainer):
         description="Mapping of task names to Simulations to run as a batch.",
     )
 
+    folder_name: str = pd.Field(
+        "default",
+        title="Folder Name",
+        description="Name of folder to store member of each batch on web UI.",
+    )
+
     jobs: Dict[TaskName, Job] = pd.Field(
         None,
         title="Simulations",
@@ -266,15 +239,14 @@ class Batch(WebContainer):
         "Set by ``Batch.upload``, leave as None.",
     )
 
-    folder_name: str = pd.Field(
-        "default",
-        title="Folder Name",
-        description="Name of folder to store member of each batch on web UI.",
-    )
+    @staticmethod
+    def _check_path_dir(path_dir: str) -> None:
+        """Make sure ``path_dir`` exists and create one if not."""
 
-    def run(
-        self, path_dir: str = DEFAULT_DATA_DIR, normalize_index: Optional[int] = 0
-    ) -> BatchData:
+        if not os.path.exists(path_dir):
+            os.makedirs(path_dir, exist_ok=True)
+
+    def run(self, path_dir: str = DEFAULT_DATA_DIR) -> BatchData:
         """Upload and run each simulation in :class:`Batch`.
 
         Parameters
@@ -297,27 +269,32 @@ class Batch(WebContainer):
 
         ``bach_data`` does not store all of the :class:`.SimulationData` objects in memory,
         rather it iterates over the task names
-        and loads the corresponding :class:`.SimulationData` from file.
+        and loads the corresponding :class:`.SimulationData` from file one by one.
         If no file exists for that task, it downloads it.
         """
-
-        self.upload()
+        self._check_path_dir(path_dir)
         self.start()
         self.monitor()
-        return self.load(path_dir=path_dir, normalize_index=normalize_index)
+        return self.load(path_dir=path_dir)
 
-    def upload(self) -> None:
+    @pd.validator("jobs", always=True)
+    def _upload(cls, val, values) -> None:
         """Create a series of tasks in the :class:`.Batch` and upload them to server.
 
         Note
         ----
         To start the simulations running, must call :meth:`Batch.start` after uploaded.
         """
-        self.jobs = {}
-        for task_name, simulation in self.simulations.items():
-            job = Job(simulation=simulation, task_name=task_name, folder_name=self.folder_name)
-            self.jobs[task_name] = job
-            job.upload()
+        if val is not None:
+            return val
+
+        jobs = {}
+        for task_name, simulation in values.get("simulations").items():
+            job = Job(
+                simulation=simulation, task_name=task_name, folder_name=values.get("folder_name")
+            )
+            jobs[task_name] = job
+        return jobs
 
     def get_info(self) -> Dict[TaskName, TaskInfo]:
         """Get information about each task in the :class:`Batch`.
@@ -424,12 +401,12 @@ class Batch(WebContainer):
 
     @staticmethod
     def _batch_path(path_dir: str = DEFAULT_DATA_DIR):
-        """Default path to save :class:`Batch` json file.
+        """Default path to save :class:`Batch` hdf5 file.
 
         Parameters
         ----------
         path_dir : str = './'
-            Base directory where the batch.json will be downloaded,
+            Base directory where the batch.hdf5 will be downloaded,
             by default, the current working directory.
 
         Returns
@@ -437,7 +414,7 @@ class Batch(WebContainer):
         str
             Full path to the batch file.
         """
-        return os.path.join(path_dir, "batch.json")
+        return os.path.join(path_dir, "batch.hdf5")
 
     def download(self, path_dir: str = DEFAULT_DATA_DIR) -> None:
         """Download results of each task.
@@ -452,7 +429,7 @@ class Batch(WebContainer):
         To load the data into :class:`.SimulationData`objets, can call :meth:`Batch.items`.
 
         The data for each task will be named as ``{path_dir}/{task_name}.hdf5``.
-        The :class:`Batch` json file will be automatically saved as ``{path_dir}/batch.json``,
+        The :class:`Batch` hdf5 file will be automatically saved as ``{path_dir}/batch.hdf5``,
         allowing one to load this :class:`Batch` later using ``batch = Batch.from_file()``.
         """
 
@@ -462,9 +439,7 @@ class Batch(WebContainer):
             job_path = self._job_data_path(task_id=job.task_id, path_dir=path_dir)
             job.download(path=job_path)
 
-    def load(
-        self, path_dir: str = DEFAULT_DATA_DIR, normalize_index: Optional[int] = 0
-    ) -> BatchData:
+    def load(self, path_dir: str = DEFAULT_DATA_DIR) -> BatchData:
         """Download results and load them into :class:`.BatchData` object.
 
         Parameters
@@ -477,7 +452,7 @@ class Batch(WebContainer):
         :class:`BatchData`
             Contains the :class:`.SimulationData` of each :class:`.Simulation` in :class:`Batch`.
 
-        The :class:`Batch` json file will be automatically saved as ``{path_dir}/batch.json``,
+        The :class:`Batch` hdf5 file will be automatically saved as ``{path_dir}/batch.hdf5``,
         allowing one to load this :class:`Batch` later using ``batch = Batch.from_file()``.
         """
 
@@ -492,10 +467,9 @@ class Batch(WebContainer):
             task_paths[task_name] = self._job_data_path(task_id=job.task_id, path_dir=path_dir)
             task_ids[task_name] = self.jobs[task_name].task_id
 
-        return BatchData(task_paths=task_paths, task_ids=task_ids, normalize_index=normalize_index)
+        return BatchData(task_paths=task_paths, task_ids=task_ids)
 
     def delete(self):
         """Delete server-side data associated with each task in the batch."""
         for _, job in self.jobs.items():
             job.delete()
-            self.jobs = None
